@@ -1,15 +1,15 @@
-# 
+#
 # This file is part of Config-Model-TkUI
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2011 by Dominique Dumont.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 package Config::Model::Tk::CheckListEditor ;
 BEGIN {
-  $Config::Model::Tk::CheckListEditor::VERSION = '1.317';
+  $Config::Model::Tk::CheckListEditor::VERSION = '1.319';
 }
 
 use strict;
@@ -75,11 +75,11 @@ sub Populate {
                                  ) ->pack(@fbe1) ;
     $lb->insert('end',@choice) ;
 
-
-    my $get_selected = sub { return map { $choice[$_]} $lb->curselection ;};
-
     # mastering perl/Tk page 160
-    my $b_sub = sub { $cw->set_value_help(&$get_selected);} ;
+    my $b_sub = sub { 
+        my @selected = map { $choice[$_]} $lb->curselection ;
+        $cw->set_value_help(@selected);
+    } ;
     $lb->bind('<<ListboxSelect>>',$b_sub);
 
     my $bframe = $ed_frame->Frame->pack;
@@ -93,7 +93,7 @@ sub Populate {
                         -command => sub { $cw->reset_value ; },
                       ) -> pack(-side => 'left') ;
     $bframe -> Button ( -text => 'Store',
-                        -command => sub { $cw->store ( &$get_selected )},
+                        -command => sub { $cw->store ( )},
                       ) -> pack(-side => 'left') ;
 
     $cw->ConfigModelNoteEditor( -object => $leaf )->pack(@fbe1) ;
@@ -110,6 +110,8 @@ sub Populate {
         $cw->add_change_order_page($nb,$leaf) ;
     }
 
+    $cw->Advertise('listbox' => $lb ) ;
+
     # don't call directly SUPER::Populate as it's CheckListViewer's populate
     $cw->Tk::Frame::Populate($args) ;
 }
@@ -197,7 +199,10 @@ sub move_selected_down {
 sub store {
     my $cw = shift ;
 
-    my %set = map { $_ => 1 ; } @_ ;
+    my $lb = $cw->Subwidget('listbox') ;
+    my @choice = $cw->{leaf}->get_choice ;
+    
+    my %set = map { $_ => 1 ; } map { $choice[$_]} $lb->curselection ;
     my $cl = $cw->{leaf};
 
     map {
@@ -207,7 +212,7 @@ sub store {
         elsif (not $set{$_} and $cl->is_checked($_) ) {
             $cl->uncheck($_) ;
         }
-    } $cw->{leaf}->get_choice;
+    } @choice;
 
     $cw->{store_cb}->() ;
 }
